/*********
    * 
    * CS-Rockz Software Solutions Private Limited. Confidential
    * ____
    * 
    *  Copyright CS-Rockz Software Solutions Private Limited. 2020. All rights reserved
    * 
    * NOTICE:  All information contained herein is, and remains
    * the property of CS-Rockz Software Solutions Private Limited. and its suppliers,
    * if any.  The intellectual and technical concepts contained
    * herein are proprietary to CS-Rockz Software Solutions Private Limited.
    * and its suppliers and may be covered by U.S. and Foreign Patents,
    * patents in process, and are protected by trade secret or copyright law.
    * Dissemination of this information or reproduction of this material
    * is strictly forbidden unless prior written permission is obtained
    * from CS-Rockz Software Solutions Private Limited.
*/
'use strict'
const DEFAULT_PARAMS = require("../data/default_package_params")
const fs = require('fs-extra')
const spinner = require('ora')()
const util = require('util')
const { promisify } = require('util')
const writeFileAsync = promisify(fs.writeFile)
const mkdirAsync = promisify(fs.mkdir)
const exec = util.promisify(require('child_process').exec)
const chalk = require('chalk')
const exampleApp = "workbook"

/**
 * This method imports created react library
 * @param {String} dest - Destination directory path
 * @param {*} libName - Created react library name
 */
const updateWorkbook = async (dest, libName) => writeFileAsync(`${dest}/src/App.js`, `
import Anything from "${libName}";
export default function App () {
    return <Anything />
}`)

/**
 * To update package.json using user prompt input
 * @param {String} dest - Destination directory path
 * @param {Object} params - User prompt input as object
 */
const updatePackageJson = async (dest, params) => {
    try {
        let pkg = require(`${dest}/package.json`);
        pkg.name = params.name;
        pkg.version = params.version;
        pkg.author = params.author;
        pkg.license = params.license;
        pkg.repository = params.repository;
        pkg.description = params.description;
        pkg.keywords = params.keywords.split(",");
        await writeFileAsync(`${dest}/package.json`, JSON.stringify(pkg, null, 4));

        // Creating npm ignore file
        writeFileAsync(`${dest}/.npmignore`, `
.babelrc
webpack.config.js
app.js
jsconfig.json
tsconfig.json
workbook
build/index.js.LICENSE.txt
.history
assets`)
    } catch (err) {
        console.error("Exception occurred while updating package.json -- ", err);
    }
    return;
}

/**
 * To copy user selected template
 * @param {String} dest - Destination directory path
 * @param {Object} params - User prompt input as object
 */
const copyTemplate = async (dest, params) => {
    try {
        spinner.start(`Copying template to ${dest}`);
        await fs.copy(`${__dirname}/../templates/default`, dest);
        await updatePackageJson(dest, params);
        spinner.succeed();
    } catch (err) {
        console.error("Exception occurred while copying template -- ", err);
    }
    return;
}

module.exports = async (promptParams = DEFAULT_PARAMS) => {
    try {
        let dest = `${process.cwd()}/${promptParams.name}`;
        await mkdirAsync(dest);
        await copyTemplate(dest, promptParams);

        spinner.start(`Installing dependencies - npm i`);
        await exec('npm i', { cwd: `${dest}` });
        spinner.succeed();

        spinner.start(`Creating react workbook(example) app using CRA => npx create-react-app ${exampleApp}`);
        await exec(`npx create-react-app ${exampleApp}`, { cwd: `${dest}` });
        await exec(`npm i ../../${promptParams.name}`, { cwd: `${dest}/${exampleApp}` });
        await updateWorkbook(`${dest}/${exampleApp}`, promptParams.name);
        await writeFileAsync(`${dest}/${exampleApp}/.env`, "SKIP_PREFLIGHT_CHECK=true");
        spinner.succeed();


        // Printing success messages in terminal 
        console.log(`
Your module has been created at ${dest}

To get plugin(library) started, in one terminal, run:
$ ${chalk.cyan(`cd ${promptParams.name} && npm start`)}

To get workbook started, in another terminal, run:    
$ ${chalk.cyan(`cd ${promptParams.name}/${exampleApp} && npm start`)}
`)

    } catch (err) {
        spinner.fail(`Unable to create ${promptParams.name} react library`);
        throw err;
    }
}